def getQuality(qualityData, rawbase, start, end, sequenceName):
    ###### Get Quality Input
    #print("sequenceName : " + sequenceName)
    #print("rawbase : " + str(rawbase))
    #print("start : " + str(start))
    #print("end : " + str(end))


    minBaseOcurrances = 99999999999
    minBasePercentages = 100.0
    minBaseQualityPercentage = 100.0

    arrBases = list(rawbase)
    i = start
    for amplicon in qualityData['Amplicon']:
        for allele in amplicon['Allele']:
            if (allele['SequenceName'] == sequenceName):
                #for i in range(start, end):
                for base in arrBases:
                    if(base == '-'):
                        base = "Deletion"

                    for k, v in allele['BaseOcurrances'].items():
                        if (k == base):
                            x = v.split(",")
                            #print(x[i])
                            if (minBaseOcurrances > int(x[i]) ):
                                minBaseOcurrances = int(x[i])
                            break
                    for k, v in allele['BasePercentages'].items():
                        if (k == base):
                            x = v.split(",")
                            #print(x[i])
                            if (minBasePercentages > float(x[i])):
                                minBasePercentages = float(x[i])
                            break
                    for k, v in allele['BaseQualityPercentage'].items():
                        if (k == base):
                            x = v.split(",")
                            #print(x[i])
                            if (minBaseQualityPercentage > float(x[i])):
                                minBaseQualityPercentage = float(x[i])
                            break

                    i=i+1

    return minBaseOcurrances,minBasePercentages,minBaseQualityPercentage



import bamnostic as bs
import argparse
import os.path
import json
import csv

from WholeGenomeAnalysis.SNP.model.AutoTypingResult import AutoTypingResult
from WholeGenomeAnalysis.SNP.model.Sample import Sample
from WholeGenomeAnalysis.SNP.model.Amplicon import Amplicon
from WholeGenomeAnalysis.SNP.model.SequenceObject import SequenceObject
from WholeGenomeAnalysis.SNP.model.Snp import Snp
from WholeGenomeAnalysis.SNP.process import CallSnip as callsnip
from WholeGenomeAnalysis.SNP.process import QualityCalculation as qualityCalculation
from WholeGenomeAnalysis.SNP.reader import reader as reader
from WholeGenomeAnalysis.SNP.writer import writer as write
# from WholeGenomeAnalysis.SNP.model.Vcf import Vcf
from WholeGenomeAnalysis.SNP.process import runConvert
from pathlib import Path

parser = argparse.ArgumentParser(description='Input for Snip program')

parser.add_argument("-s", "--snpJsonpath", help='input snp json file path', required=True, dest="snpJsonpath")
parser.add_argument("-j", "--jsonpath", help='input json file path (output json from the sample processor program', required=True, dest="jsonPath")
parser.add_argument("-b", "--bampath", help='input bam file path', required=True, dest="bamPath")
parser.add_argument("-o", "--outfolder", help='output folder location', required=True, dest="outfolder")
parser.add_argument("-v", "--vcf", help='vcf head information path', required=True, dest="vcfpath")

complement = {'A': 'T', 'C': 'G', 'G': 'C', 'T': 'A'}

args = parser.parse_args()

if (os.path.exists(args.snpJsonpath)):
    print("File exist : " + args.snpJsonpath)
else:
    print("Missing json file : " + args.snpJsonpath)
    exit()

if (os.path.exists(args.bamPath)):
    print("File exist : " + args.bamPath)
else:
    print("Missing bam file : " + args.bamPath)
    exit()

if (os.path.exists(args.jsonPath)):
    print("File exist : " + args.jsonPath)
else:
    print("Missing bam json : " + args.jsonPath)
    exit()

if (os.path.exists(args.outfolder)):
    print("Folder exist : " + args.outfolder)
else:
    print("Created path : " + args.outfolder)
    os.mkdir(args.outfolder)

if (os.path.exists(args.vcfpath)):
    print("File exist : " + args.vcfpath)
else:
    print("Missing vcf header file : " + args.vcfpath)
    exit()


snpJsonpath = args.snpJsonpath
bamPath = args.bamPath
jsonpath = args.jsonPath

snpData = reader.JsonReader(snpJsonpath)
qualityData = reader.JsonReader(jsonpath)
bam = bs.AlignmentFile(bamPath, 'rb')

# copiedbam = copy.copy(bam)

# for snp in data['SNP']:
#     print('name : ' + snp['name'])

snpVersion = snpData['SNPVERSION']
targetSnpVersion = snpData['VERSION']
experimentID = qualityData['ExperimentID']
experimentName = qualityData['ExperimentName']
plateName = qualityData['PlateName']
sampleId = qualityData['SampleID']


# Create output path

#outputpath = args.outfolder+'/'+sampleId+'.csv'
#outputpath_vcf = args.outfolder+'/'+sampleId+'.vcf'

outputpath = Path(args.outfolder+'/'+sampleId+".csv")
outputpath_vcf = Path(args.outfolder+'/'+sampleId+".vcf")

print("output csv path : " + str(outputpath))
print("output vcf path : " + str(outputpath_vcf))

autoResult = AutoTypingResult(snpVersion, targetSnpVersion)
smpl = Sample(experimentID, experimentName, plateName, sampleId)

snip_dic = {}
snip_totalReads = {}

#with open(outputpath, mode='w', newline='') as csv_file:
#    fieldnames = ['experimentName', 'sampleId', 'seqName', 'rawReads', 'snpName', 'chrom', 'chromStart', 'chromEnd',
#                  'refNCBI', 'refUCSC', 'targeted', 'observed', 'minBaseOcurrances', 'minBasePercentages',
#                  'minBaseQualityPercentage']
#    writer = csv.DictWriter(csv_file, fieldnames=fieldnames)
#    writer.writeheader()

for sequence in bam:
    if (hasattr(sequence, 'reference_name')):
        #if sequence.query_name.startswith('SNP'):
        arrObject = []

        start = sequence.pos
        end = sequence.pos + len(sequence.seq)
        if(sequence.flag == 16):
            strand = '-'
        else:
            strand = '+'

        sObject = SequenceObject(start,end,sequence.cigar,sequence.seq,sequence.reference_name)
        arrObject.append(sObject)

        xa=''
        if "XA" in sequence.tags:
            xa = sequence.tags.get("XA")[1]

        if(";" in xa):
            pair_list = xa.split(";")
            for pair in pair_list:
                if(pair != ''):
                    comma_list = pair.split(",")
                    newReferenceName=comma_list[0]
                    newstrand=comma_list[1][0:1]
                    newstart=int(comma_list[1][1:])
                    newend=int(comma_list[1][1:]) + len(sequence.seq)
                    newcigar=bs.utils.parse_cigar(comma_list[2])

                    if(strand == newstrand):
                        newSequence = sequence.seq
                    else:
                        newSequence = "".join(complement.get(base,base) for base in reversed(sequence.seq))

                    arrObject.append(SequenceObject(newstart,newend,newcigar,newSequence,newReferenceName))

        # print("1. reference_name: " + sequence.reference_name + " | 1. sequence.query_name: " + sequence.query_name)
        # print("1. sequence.read_name: " + sequence.read_name)
        # dt = {}
        #print("Name : " + sequence.query_name)
        AmpliconName = sequence.query_name.split('#')[2]
        amp = Amplicon(AmpliconName)

        for matchObject in arrObject:
            matchStart, matchEnd, matchCigar, matchSeq, matchChr = matchObject.get()

            for snip in snpData['SNP']:
                if (snip['chrom'] == matchChr):

                    # print("2. reference_name: " + sequence.reference_name)v
                    # print("2. sequence.query_name: " + sequence.query_name)

                    chromStart = snip['chromStart']
                    chromEnd = snip['chromEnd']

                    if (matchStart < chromStart < matchEnd and matchStart < chromEnd < matchEnd):
                        # print("1. reference_name: " + sequence.reference_name + " | 1. sequence.query_name: " + sequence.query_name)

                        # AmpliconName = sequence.query_name.split('#')[2]
                        insertionCount = callsnip.CigarInsertionCount(matchCigar)
                        if (insertionCount > 10):
                            continue

                        rawReads = sequence.query_name.split('#')[4]
                        targetPositionStart = chromStart - matchStart
                        targetPositionEnd = chromEnd - matchStart
                        #print("targetPositionStart : " + str(targetPositionStart) + " | targetPositionEnd: " + str(targetPositionEnd))
                        calStart, calend, baseMark= callsnip.CigarCalculation(targetPositionStart, targetPositionEnd, matchCigar)
                        #print("start : " + str(calStart) + " | end: " + str(calend))
                        rawbases = matchSeq[calStart:calend]

                        if (baseMark == 'D'):
                            rawbases = '-'

                        if (calStart == calend and rawbases == ''):
                            rawbases = '-'

                        minBaseOcurrances, minBasePercentages, minBaseQualityPercentage = qualityCalculation.getQuality(qualityData, rawbases, calStart, calend, sequence.query_name)
                        #rawbases = sequence.seq[targetPositionStart:targetPositionEnd]

                        #snps = Snp(1, sequence.query_name, rawReads, snip['name'], snip['chrom'], chromStart, chromEnd,
                        #           snip['refNCBI'], snip['refUCSC'], snip['observed'], rawbases, minBaseOcurrances, minBasePercentages, minBaseQualityPercentage, sequence)

                        # dt.update(vars(snps))
                        # dt = json.dumps(dt)

                        #amp.add(snps)

                        snps = Snp(1, sequence.query_name, rawReads, snip['name'], snip['chrom'].replace("chr", ""), str(chromStart), str(chromEnd),
                                   snip['refNCBI'], snip['refUCSC'], snip['observed'], rawbases, str(minBaseOcurrances), str(minBasePercentages), str(minBaseQualityPercentage), sequence)

                        if(snip['name'] in snip_dic):
                            dicSnipDetail = snip_dic[snip['name']]
                            if(rawbases in dicSnipDetail):
                                getSnip = dicSnipDetail[rawbases]

                                getSnip.rawReads = int(getSnip.rawReads)+int(snps.rawReads)
                                getSnip.minBaseOcurrances = int(getSnip.minBaseOcurrances) + int(snps.minBaseOcurrances)

                                if (getSnip.minBasePercentages > snps.minBasePercentages):
                                    getSnip.minBasePercentages = snps.minBasePercentages
                                if(getSnip.minBasePercentages > snps.minBasePercentages):
                                    getSnip.minBasePercentages = snps.minBasePercentages
                                if (getSnip.minBaseQualityPercentage > snps.minBaseQualityPercentage):
                                    getSnip.minBaseQualityPercentage = snps.minBaseQualityPercentage

                                #lstSnip.append(snps)
                                dicSnipDetail[rawbases] = getSnip
                            else:
                                #lstSnip = []
                                #lstSnip.append(snps)
                                dicSnipDetail[rawbases] = snps
                        else:
                            #lstSnip = []
                            dicSnipDetail = {}
                            #lstSnip.append(snps)
                            dicSnipDetail[rawbases]=snps
                            snip_dic[snip['name']] = dicSnipDetail

                        if( snip['name'] in snip_totalReads ):
                            snip_totalReads[snip['name']] = int(snip_totalReads[snip['name']])+int(rawReads)
                        else:
                            snip_totalReads[snip['name']] = int(rawReads)


                        #writer.writerow({'experimentName': experimentName, 'sampleId': sampleId, 'seqName': sequence.query_name, 'rawReads': rawReads, 'snpName': snip['name'], 'chrom': snip['chrom'], 'chromStart': str(chromStart), 'chromEnd': str(chromEnd), 'refNCBI': snip['refNCBI'], 'refUCSC': snip['refUCSC'], 'targeted': snip['observed'], 'observed': rawbases, 'minBaseOcurrances': str(minBaseOcurrances), 'minBasePercentages': str(minBasePercentages), 'minBaseQualityPercentage': str(minBaseQualityPercentage)})

                        print(experimentName + ',' + sampleId + ',' + sequence.query_name + ',' + rawReads + ',' + snip[
                            'name'] + ',' + snip['chrom'].replace("chr", "") + ',' + str(chromStart) + ',' + str(chromEnd) + ',' + snip[
                                  'refNCBI'] + ',' + snip['refUCSC'] + ',' + snip['observed'] + ',' + rawbases  + ',' + str(minBaseOcurrances) + ',' + str(minBasePercentages) + ',' + str(minBaseQualityPercentage))
                        isTargeted = True

            # AmpliconName = sequence.query_name.split('#')[2]
            # amp = Amplicon(AmpliconName)

            # if AmpliconName in AmpliconList:
            #     amp = AmpliconList[AmpliconName] # Get Amplicon object by AmpliconName
            #     amp.add(dt)

            #if isTargeted == True:
            #    smpl.add(amp)
            #else:
            #    print("None Target List: " + sequence.query_name)

final_snp_list = callsnip.filterSnipObject(snip_dic, snip_totalReads,0.05)
write.csvWriter(outputpath, final_snp_list, experimentName,sampleId)


converter = runConvert.Converter()
vcfs = converter.convert(final_snp_list, sampleId)
header = reader.VCFReader(args.vcfpath, sampleId)

write.TextWriter(outputpath_vcf, header)
write.VCFWriter(outputpath_vcf, vcfs)





# autoResult.add(smpl)
#
# autoResult.autoTyping_dic = {}
# autoResult.autoTyping_dic["snpVersion"] = autoResult.snpVersion
# autoResult.autoTyping_dic["targetSnpVersion"] = autoResult.targetSnpVersion
# autoResult.autoTyping_dic["sample"] = autoResult.sample_list
#
# Convert_Json = {
#     "autoTypingResult": autoResult.autoTyping_dic
# }
#
# jsonFormat = json.dumps(Convert_Json, indent=4)
# print(jsonFormat)




import copy


class AutoTypingResult:

    def __init__(self, snpVersion, targetSnpVersion):
        self.snpVersion = snpVersion
        self.targetSnpVersion = targetSnpVersion
        self.Sample = []
        self.sample_dic = {}
        self.sample_list = []

    def get(self):
        return self.Sample

    def add(self, Sample):
        self.Sample.append(Sample)
        self.sample_dic = copy.deepcopy(self.sample_dic)
        self.sample_dic["experimentID"] = Sample.experimentID
        self.sample_dic["experimentName"] = Sample.experimentName
        self.sample_dic["plateName"] = Sample.plateName
        self.sample_dic["sampleId"] = Sample.sampleId
        self.sample_dic["amplicon"] = Sample.amplicon_list
        self.sample_list.append(self.sample_dic)

    def reprJSON(self):
        return dict(snpVersion=self.snpVersion, targetSnpVersion=self.targetSnpVersion, Sample=self.Sample)

    # def make_autoTypingResult(snpVersion, targetSnpVersion):
#     autoTypingResult = AutoTypingResult(snpVersion, targetSnpVersion)
#     return autoTypingResult



# from WholeGenomeAnalysis.SNP.reader import reader
# import os
# import bamnostic as bs

from GenomeAnalysis.WholeGenomeAnalysis.SNP.model.AutoTypingResult import AutoTypingResult
from GenomeAnalysis.WholeGenomeAnalysis.SNP.model.Sample import Sample
from GenomeAnalysis.WholeGenomeAnalysis.SNP.model.Amplicon import Amplicon
from GenomeAnalysis.WholeGenomeAnalysis.SNP.model.Snp import Snp

# for j in range(1, 3):
#     venue1 = Amplicon()
#     venue1.set("rname" + str(j))
#
#     print('j' + str(j))
#
#     for i in range(j):
#         w1 = Snp(i, "seqName", 100, "SNP_0", "chrom", 1000, 1002, "refNCBI", "refUCSC", "observed", "AT", 100, 100, 95,"ACGTACGT")
#         w1.seqName = "SNP_" + str(i)
#         venue1.add(w1)
#         print('i' + str(i))
#
#     print(vars(venue1))
#     for sp in venue1.get():
#         print(vars(sp))


w1 = Snp(1, "seqName", 100, "SNP_0", "chrom", 1000, 1002, "refNCBI", "refUCSC", "observed", "AT", 100, 100, 95, "ACGTACGT")
w2 = Snp(1, "seqName", 100, "SNP_0", "chrom", 1000, 1002, "refNCBI", "refUCSC", "observed", "AT", 100, 100, 95, "ACGTACGT")
w3 = Snp(1, "seqName", 100, "SNP_0", "chrom", 1000, 1002, "refNCBI", "refUCSC", "observed", "AT", 100, 100, 95, "ACGTACGT")

w1.seqName = "SNP_1"
w2.seqName = "SNP_2"
w3.seqName = "SNP_3"

v1 = Amplicon("rname1")
# v1.set("rname1")

v1.add(w1)
v1.add(w2)

v2 = Amplicon("rname2")
# v2.set("rname2")

v2.add(w3)


x1 = Sample("experimentID", "experimentName", "plateName", "S1--S1")
x1.add(v1)

x2 = Sample("experimentID", "experimentName", "plateName", "S2--S2")
x2.add(v2)

y = AutoTypingResult("snpVersion", "targetSnpVersion")
y.add(x1)
y.add(x2)

print(vars(y))
for sample in y.get():
    print(vars(sample))

    for amplicon in sample.get():
        print(vars(amplicon))

        for snp in amplicon.get():
            print(vars(snp))


# print(vars(v1))
# for sp in v1.get():
#     print(vars(sp))
#
# print('------------- 1')
#
# print(vars(v2))
# for sp in v2.get():
#     print(vars(sp))
#
# print('------------- 2')

# filepath=r'C:\Users\seho\PycharmProjects\GenomeAnalysis\Documentation\SNP151.json'
# bampath=r'C:\Users\seho\PycharmProjects\GenomeAnalysis\Example\284169513\S284169513.bam'
# data=reader.snpJsonReader(filepath)
#
# for snp in data['SNP']:
#     print('name : ' + snp['name'])
#
# bam=bs.AlignmentFile(bampath, 'rb')
#
# #for sequence in bam:
# #    if(hasattr(sequence, 'reference_name')):
# #        print(sequence.reference_name)
#
# BlotName='NMP'
# SampleName='284169513'
# for sequence in bam:
#     if (hasattr(sequence, 'reference_name')):
#         start=sequence.pos
#         end=sequence.pos+len(sequence.seq)
#         for snp in data['SNP']:
#             if(snp['chrom'] == sequence.reference_name):
#                 chromStart=snp['chromStart']
#                 chromEnd=snp['chromEnd']
#                 if(start < chromStart < end and start < chromEnd < end):
#                     AmpliconName=sequence.query_name.split('#')[2]
#                     rawReads = sequence.query_name.split('#')[4]
#                     targetPositionStart=chromStart-start
#                     targetPositionEnd=chromEnd-start
#
#                     rawbases=sequence.seq[targetPositionStart:targetPositionEnd]
#                     print(BlotName+','+SampleName+','+AmpliconName+','+rawReads+','+snp['name']+','+snp['chrom']+','+str(chromStart)+','+str(chromEnd)+','+snp['refNCBI']+','+snp['refUCSC']+','+snp['observed']+','+rawbases)







class SequenceObject:
    def __init__(self, start,end,cigar,seq,chr):
        self.start = start
        self.end = end
        self.cigar = cigar
        self.seq = seq
        self.chr = chr

    def get(self):
        return self.start,self.end,self.cigar,self.seq,self.chr



import csv

def TextWriter(outputpath_vcf, data):
    file = open(outputpath_vcf, 'w')
    for dt in data:
        file.writelines(dt)

    # with open(outputpath_vcf, mode='w') as result:
    #     work = csv.writer(result, delimiter='\t', quoting=csv.QUOTE_MINIMAL)
    #     work.writerow(data)

def csvWriter(outputpath_csv, snip_dic,experimentName,sampleId):
    with open(outputpath_csv, mode='w', newline='') as csv_file:
        #fieldnames = ['experimentName', 'sampleId', 'snpName', 'observed1', 'observed2', 'observed3', 'observed4']
        fieldnames = ['experimentName', 'sampleId', 'snpName', 'observed1', 'reads1','observed2', 'reads2', 'observed3', 'reads3','observed4', 'reads4']
        writer = csv.DictWriter(csv_file, fieldnames=fieldnames)
        writer.writeheader()

        for snip, snip_observe  in snip_dic.items():
            observe1= ''
            reads1=''
            observe2= ''
            reads2=''
            observe3= ''
            reads3=''
            observe4= ''
            reads4=''
            count=0
            for observe in sorted(snip_observe.keys()):
                snp = snip_observe[observe]
                if(snp.isValid != 0):
                    if(count==0):
                        observe1=observe
                        reads1=snp.rawReads
                    elif(count==1):
                        observe2=observe
                        reads2 = snp.rawReads
                    elif(count==2):
                        observe3=observe
                        reads3 = snp.rawReads
                    elif(count==3):
                        observe4=observe
                        reads4 = snp.rawReads

                    count=count+1

            writer.writerow(
                {'experimentName': experimentName, 'sampleId': sampleId, 'snpName': snip, 'observed1': observe1, 'reads1' : reads1,
                 'observed2': observe2,'reads2' : reads2,'observed3': observe3,'reads3' : reads3,'observed4': observe4,'reads4' : reads4,})


    # with open(outputpath_vcf, mode='w') as result:
    #     work = csv.writer(result, delimiter='\t', quoting=csv.QUOTE_MINIMAL)
    #     work.writerow(data)

def VCFWriter(outputpath_vcf, vcfs):

    file = open(outputpath_vcf, 'a')

    for vcf in vcfs:
        format = ""
        format_value = ""
        data = []

        data.append(vcf.chrom)
        data.append(vcf.pos)
        data.append(vcf.id)
        data.append(vcf.ref)
        # data.append(vcf.alt[:len(vcf.alt) - 1])
        data.append(vcf.alt)
        data.append(vcf.qual)
        data.append(vcf.filter)

        if vcf.alt == '<CNV>':
            value = ""
            for name in vcf.info.dic:
                value = value + ";" + name + "=" + str(vcf.info.dic[name])
            data.append(value[1:])
        else:
            for name in vcf.info.dic:
                data.append(name + "=" + str(vcf.info.dic[name]))

        for name in vcf.format.dic:
            format = format + name + ":"

        data.append(format[:len(format)-1])

        for name in vcf.format.dic:
            format_value = format_value + str(vcf.format.dic[name]) + ":"

        data.append(format_value[:len(format_value)-1])

        line = '\t#'.join(str(x) for x in data)
        lst = line.split('#')
        file.writelines(lst)
        file.writelines('\n')


import copy

class Output:

    def __init__(self, isValid, seqName, snpName, chrom, chromStart, chromEnd, refNCBI, refUCSC, observed, rawBases1 , rawBases2, rawReads1, rawReads2, minBaseOcurrances, minBasePercentages, minBaseQualityPercentage, sequence):
        self.isValid = isValid
        self.seqName = seqName
        self.rawReads1 = rawReads1
        self.rawReads2 = rawReads2
        self.snpName = snpName
        self.chrom = chrom
        self.chromStart = chromStart
        self.chromEnd = chromEnd
        self.refNCBI = refNCBI
        self.refUCSC = refUCSC
        self.observed = observed
        self.rawBases1 = rawBases1
        self.rawBases2 = rawBases2
        self.minBaseOcurrances = minBaseOcurrances
        self.minBasePercentages = minBasePercentages
        self.minBaseQualityPercentage = minBaseQualityPercentage
        self.sequence = sequence
def make_Output(isValid, seqName, snpName, chrom, chromStart, chromEnd, refNCBI, refUCSC, observed, rawBases1, rawBases2,rawReads1, rawReads2, minBaseOcurrances, minBasePercentages, minBaseQualityPercentage, sequence):
    Output = Output(isValid, seqName, rawReads, snpName, chrom, chromStart, chromEnd, refNCBI, refUCSC, observed, rawBases, minBaseOcurrances, minBasePercentages, minBaseQualityPercentage, sequence)
    return snp



import copy


class Amplicon:

    def __init__(self, ampliconName):
        self.ampliconName = ampliconName
        self.Snp = []
        self.snp_dic = {}
        self.snp_list = []

    def get(self):
        return self.Snp

    def add(self, Snp):
        self.Snp.append(Snp)
        self.snp_dic = copy.deepcopy(self.snp_dic)
        self.snp_dic["isValid"] = Snp.isValid
        self.snp_dic["seqName"] = Snp.seqName
        self.snp_dic["rawReads"] = Snp.rawReads
        self.snp_dic["snpName"] = Snp.snpName
        self.snp_dic["chrom"] = Snp.chrom
        self.snp_dic["chromStart"] = Snp.chromStart
        self.snp_dic["chromEnd"] = Snp.chromEnd
        self.snp_dic["refNCBI"] = Snp.refNCBI
        self.snp_dic["refUCSC"] = Snp.refUCSC
        self.snp_dic["observed"] = Snp.observed
        self.snp_dic["rawBases"] = Snp.rawBases
        self.snp_dic["minBaseOcurrances"] = Snp.minBaseOcurrances
        self.snp_dic["minBasePercentages"] = Snp.minBasePercentages
        self.snp_dic["minBaseQualityPercentage"] = Snp.minBaseQualityPercentage
        self.snp_dic["sequence"] = Snp.sequence
        self.snp_list.append(self.snp_dic)

    def reprJSON(self):
        return dict(ampliconName=self.ampliconName, Snp=self.Snp)

    # def make_amplicon(ampliconName):#rname
#     amplicon = Amplicon(ampliconName)#rname
#     return amplicon



import copy

class Snp:

    def __init__(self, isValid, seqName, rawReads, snpName, chrom, chromStart, chromEnd, refNCBI, refUCSC, observed, rawBases, minBaseOcurrances, minBasePercentages, minBaseQualityPercentage, sequence):
        self.isValid = isValid
        self.seqName = seqName
        self.rawReads = rawReads
        self.snpName = snpName
        self.chrom = chrom
        self.chromStart = chromStart
        self.chromEnd = chromEnd
        self.refNCBI = refNCBI
        self.refUCSC = refUCSC
        self.observed = observed
        self.rawBases = rawBases
        self.minBaseOcurrances = minBaseOcurrances
        self.minBasePercentages = minBasePercentages
        self.minBaseQualityPercentage = minBaseQualityPercentage
        self.sequence = sequence

    def reprJSON(self):
        return dict(isValid=self.isValid, seqName=self.seqName, rawReads=self.rawReads, snpName=self.snpName, chrom=self.chrom, chromStart=self.chromStart, chromEnd=self.chromEnd
                    ,refNCBI=self.refNCBI, refUCSC=self.refUCSC, observed=self.observed, rawBases=self.rawBases, minBaseOcurrances=self.minBaseOcurrances
                    , minBasePercentages=self.minBasePercentages, minBaseQualityPercentage=self.minBaseQualityPercentage, sequence=self.sequence)

def make_snp(isValid, seqName, rawReads, snpName, chrom, chromStart, chromEnd, refNCBI, refUCSC, observed, rawBases, minBaseOcurrances, minBasePercentages, minBaseQualityPercentage, sequence):
    snp = Snp(isValid, seqName, rawReads, snpName, chrom, chromStart, chromEnd, refNCBI, refUCSC, observed, rawBases, minBaseOcurrances, minBasePercentages, minBaseQualityPercentage, sequence)
    return snp



import copy

class Vcf:

    def __init__(self, chrom, pos, id, ref, alt, qual, filter, info, format, sampleid):
        self.chrom = chrom
        self.pos = pos
        self.id = id
        self.ref = ref
        self.alt = alt
        self.qual = qual
        self.filter = filter
        self.info = info
        self.format = format
        self.sampleid = sampleid


# import vcf
# import copy
# import time
from WholeGenomeAnalysis.SNP.model.Vcf import Vcf
# from WholeGenomeAnalysis.SNP.model.Info import INFO
# from WholeGenomeAnalysis.SNP.model.Format import FORMAT

# class INFO:
#     # DP = {}
#     # SVTYPE = {}
#     # FUNC = {}
#     dic = {}
#     # The class "constructor" - It's actually an initializer
#     # def __init__(self, DP, SVTYPE, FUNC):
#     #     self.DP = DP
#     #     self.SVTYPE = SVTYPE
#     #     self.FUNC = FUNC
#     def make_INFO(self, dp):
#         self.dic["DP"] = dp
#         # info = INFO(DP)
#         return INFO
#
#     def make_INFO_CNV(self):
#         self.dic["SVTYPE"] = "CNV"
#         # info = INFO(SVTYPE)
#         return INFO
#
# class FORMAT:
#     # GT = {}
#     # CN = {}
#     # DP = {}
#     dic = {}
#     # The class "constructor" - It's actually an initializer
#     # def __init__(self, GT, CN, DP):
#     #     self.GT = GT
#     #     self.CN = CN
#     #     self.DP = DP
#     def make_FORMAT_CNV(self, gt, cn):
#         self.dic["GT"] = gt
#         self.dic["CN"] = cn
#         # format = FORMAT(GT, CN)
#         return FORMAT
#
#     def make_FORMAT(self, gt, dp):
#         self.dic["GT"] = gt
#         self.dic["DP"] = dp
#         # format = FORMAT(GT, DP)
#         return FORMAT

# def make_INFO(self, dp):
#     DP = {"DP": dp}
#     info = INFO(DP)
#     return info
#
# def make_INFO_CNV(self):
#     SVTYPE = {"SVTYPE": 'CNV'}
#     info = INFO(SVTYPE)
#     return info

# def make_FORMAT_CNV(self, gt, cn):
#     GT = {"GT": gt}
#     CN = {"CN": cn}
#     format = FORMAT(GT, CN)
#     return format
#
# def make_FORMAT(self, gt, dp):
#     GT = {"GT": gt}
#     DP = {"DP": dp}
#     format = FORMAT(GT, DP)
#     return format

class Converter:
    vcfs = []

    def __init__(self):
        self.vcfAddress = "C:/Destination/vcf_sample.vcf"
        # self.vcfs = []
        self.sampleid = ''

    # def init_vcf(self):
    #     vcf_reader = vcf.Reader(open(self.vcfAddress, 'r'))
    #
    #     for record in vcf_reader:
    #         print(record)

    def set_gt(self, gt, numberofsnp):
        _gt = ''
        if numberofsnp == 1:
            if gt == 0:
                _gt = '0/0'
            elif gt == 1:
                _gt = '1/1'

        else:
            if gt == 0:
                _gt = 'ERROR'
            elif gt == 1:
                _gt = '0/1'
            elif gt == 2:
                _gt = '1/2'

        return _gt

    def convert(self, Snps, sampleId):
        self.sampleid = sampleId
        chrom = ''
        pos = ''
        end = ''
        id = ''
        ref = ''
        alt = ''
        gt = 0
        numberofsnp = 0
        reads = 0
        snpname = ''

        for name in sorted(Snps):
            for base, snp in Snps[name].items():

                if len(snpname) == 0:
                    snpname = name

                # If snp name changes, go to add last values to vcfs(list)
                if snpname != name:
                    self.vcfs, alt, gt, numberofsnp, reads, snpname = add_vcfs(self, self.vcfs, name, chrom, pos, id, ref, alt, reads, gt, numberofsnp, end)

                numberofsnp = numberofsnp + 1
                chrom = snp.chrom
                pos = snp.chromStart
                end = snp.chromEnd
                id = snp.snpName
                ref = snp.refNCBI

                if snp.rawBases == '<CNV>':
                    alt = '<CNV>'
                else:
                    alt = alt + snp.rawBases + ','

                reads = reads + int(snp.rawReads)

                if snp.refNCBI != snp.rawBases:
                    gt = gt + 1

        # If last snp, go to add values to vcfs(list) because it will not be processed above.
        else:
            self.vcfs, alt, gt, numberofsnp, reads, snpname = add_vcfs(self, self.vcfs, name, chrom, pos, id, ref, alt, reads, gt, numberofsnp, end)

        return self.vcfs

def add_vcfs(self, vcfs, name, chrom, pos, id, ref, alt, reads, gt, numberofsnp, end):

    class INFO:
        dic = {}

        def make_INFO(self, dp):
            self.dic["DP"] = dp
            # info = INFO(DP)
            return INFO

        def make_INFO_CNV(self, begin, end):
            self.dic["SVTYPE"] = "CNV"
            self.dic["END"] = end
            self.dic["SVLEN"] = end - begin
            # info = INFO(SVTYPE)
            return INFO

    class FORMAT:
        dic = {}

        def make_FORMAT_CNV(self, reads):
            self.dic["CN"] = reads
            # format = FORMAT(GT, CN)
            return FORMAT

        def make_FORMAT(self, gt, dp):
            self.dic["GT"] = gt
            self.dic["DP"] = dp
            # format = FORMAT(GT, DP)
            return FORMAT

    genotype = self.set_gt(gt, numberofsnp)
    format = FORMAT()
    info = INFO()

    if alt == '<CNV>':
        format = format.make_FORMAT_CNV(reads)
        info = info.make_INFO_CNV(pos, end)
    else:
        format = format.make_FORMAT(genotype, reads)
        info = info.make_INFO(reads)
        alt = alt[:len(alt) - 1]

    if "," in alt:
        alt = alt.replace(ref + ",", "").replace("," + ref, "")

    qual = '.'
    filter = 'PASS'

    print(chrom, pos, id, ref, alt, qual, filter, info.dic, format.dic, self.sampleid)

    vcfs.append(Vcf(chrom, pos, id, ref, alt, qual, filter, info, format, self.sampleid))

    # re-initialize
    alt = ''
    gt = 0
    numberofsnp = 0
    reads = 0
    snpname = name

    return vcfs, alt, gt, numberofsnp, reads, snpname

# def run():
#     start = time.time()
#     # Initiate
#     vcf = Converter()
#     # Set HEADER on VCF
#     # vcf.init_vcf()
#
#     end = time.time()
#     print('Comeplete! Total time is ')
#     print(end - start)
#
# if __name__ == '__main__':
#     run()



import bamnostic as bs
import argparse
import os.path
import json
import csv

from WholeGenomeAnalysis.SNP.model.AutoTypingResult import AutoTypingResult
from WholeGenomeAnalysis.SNP.model.Sample import Sample
from WholeGenomeAnalysis.SNP.model.Amplicon import Amplicon
from WholeGenomeAnalysis.SNP.model.SequenceObject import SequenceObject
from WholeGenomeAnalysis.SNP.process import CallSnip as callsnip
from WholeGenomeAnalysis.SNP.model.Snp import Snp
from WholeGenomeAnalysis.SNP.process import QualityCalculation as qualityCalculation
from WholeGenomeAnalysis.SNP.reader import reader as reader
from pathlib import Path
from WholeGenomeAnalysis.SNP.writer import writer as write
# from WholeGenomeAnalysis.SNP.model.Vcf import Vcf
from WholeGenomeAnalysis.SNP.process import runConvert

parser = argparse.ArgumentParser(description='Input for Snip program')

parser.add_argument("-s", "--snpJsonpath", help='input snp json file path', required=True, dest="snpJsonpath")
parser.add_argument("-j", "--jsonpath", help='input json file path (output json from the sample processor program', required=True, dest="jsonPath")
parser.add_argument("-b", "--bampath", help='input bam file path', required=True, dest="bamPath")
parser.add_argument("-o", "--outfolder",help='output folder location', required=True, dest="outfolder")
parser.add_argument("-v", "--vcf", help='vcf head information path', required=True, dest="vcfpath")
parser.add_argument("-t", "--trimPercent", help='trim percentage within the sequence', required=True, dest="trimPercent")
parser.add_argument("-c", "--cnvDataPath", help='CNV Json file required as input', required=True, dest="cnvDataPath")

complement = {'A': 'T', 'C': 'G', 'G': 'C', 'T': 'A'}

args = parser.parse_args()

if (os.path.exists(args.snpJsonpath)):
    print("File exist : " + args.snpJsonpath)
else:
    print("Missing json file : " + args.snpJsonpath)
    exit()

if (os.path.exists(args.bamPath)):
    print("File exist : " + args.bamPath)
else:
    print("Missing bam file : " + args.bamPath)
    exit()

#if (os.path.exists(args.jsonPath)):
#    print("File exist : " + args.jsonPath)
#else:
#    print("Missing bam json : " + args.jsonPath)
#    exit()

#os.mkdir(args.outfolder, exist_ok=True)


if (os.path.exists(args.outfolder)):
    print("Folder exist : " + args.outfolder)
else:
    print("Created path : " + args.outfolder)
    os.mkdir(args.outfolder)

if (os.path.exists(args.vcfpath)):
    print("File exist : " + args.vcfpath)
else:
    print("Missing vcf header file : " + args.vcfpath)
    exit()

snpJsonpath = args.snpJsonpath
bamPath = args.bamPath
jsonpath = args.jsonPath
cnvPath = args.cnvDataPath
trimPercent = float(args.trimPercent)

snpData = reader.JsonReader(snpJsonpath)
experimentData = reader.JsonReader(jsonpath)
cnvData = reader.JsonReader(cnvPath)
bam = bs.AlignmentFile(bamPath, 'rb')

# copiedbam = copy.copy(bam)

# for snp in data['SNP']:
#     print('name : ' + snp['name'])

snpVersion = snpData['SNPVERSION']
targetSnpVersion = snpData['VERSION']
jobName = experimentData['jobName']
experimentName = experimentData['experimentName']
sampleID = experimentData['sampleID']
barcodeID = experimentData['barcodeID']
experimentID = experimentData['experimentID']

# Create output path

#outputpath = args.outfolder+'/'+"par"+'.csv'
outputpath = Path(args.outfolder+'/'+sampleID+".csv")
outputpath_vcf = Path(args.outfolder+'/'+sampleID+".vcf")

print("output csv path : " + str(outputpath))
print("output vcf path : " + str(outputpath_vcf))
#outputfile = writer.csvWriter(outputpath)


# sorted_bam = sorted(bam.head(), key=operator.itemgetter(1), reverse=True)

# for reading in enumerate(bamtmp):
#     print(reading)
# Result = {}
# AmpliconList = {}

# for sequence in copiedbam:
#     if (hasattr(sequence, 'reference_name')):
#         if sequence.query_name.startswith('SNP'):
#             AmpliconName = sequence.query_name.split('#')[2]
#             AmpliconList[AmpliconName] = Amplicon(AmpliconName)

autoResult = AutoTypingResult(snpVersion, targetSnpVersion)
smpl = Sample(experimentID, experimentName, '', sampleID)

snip_dic = {}
snip_totalReads = {}


#with open(outputpath, mode='w', newline='') as csv_file:
#    fieldnames = ['experimentName', 'sampleId', 'seqName', 'rawReads', 'snpName', 'chrom', 'chromStart', 'chromEnd',
#                  'refNCBI', 'refUCSC', 'targeted', 'observed', 'minBaseOcurrances', 'minBasePercentages',
#                  'minBaseQualityPercentage']
#    writer = csv.DictWriter(csv_file, fieldnames=fieldnames)
#    writer.writeheader()

for sequence in bam:
    if (hasattr(sequence, 'reference_name')):
        #if sequence.query_name.startswith('SNP'):
            arrObject = []

            start = sequence.pos
            end = sequence.pos + len(sequence.seq)
            if(sequence.flag == 16):
                strand = '-'
            else:
                strand = '+'

            sObject = SequenceObject(start,end,sequence.cigar,sequence.seq,sequence.reference_name)
            arrObject.append(sObject)

            xa=''
            if "XA" in sequence.tags:
                xa = sequence.tags.get("XA")[1]

            if(";" in xa):
                pair_list = xa.split(";")
                for pair in pair_list:
                    if(pair != ''):
                        comma_list = pair.split(",")
                        newReferenceName=comma_list[0]
                        newstrand=comma_list[1][0:1]
                        newstart=int(comma_list[1][1:])
                        newend=int(comma_list[1][1:]) + len(sequence.seq)
                        newcigar=bs.utils.parse_cigar(comma_list[2])

                        if(strand == newstrand):
                            newSequence = sequence.seq
                        else:
                            newSequence = "".join(complement.get(base,base) for base in reversed(sequence.seq))

                        arrObject.append(SequenceObject(newstart,newend,newcigar,newSequence,newReferenceName))

            # print("1. reference_name: " + sequence.reference_name + " | 1. sequence.query_name: " + sequence.query_name)
            # print("1. sequence.read_name: " + sequence.read_name)
            # dt = {}
            print("Name : " + sequence.query_name)
            AmpliconName = "CYP"
            amp = Amplicon(AmpliconName)

            for matchObject in arrObject:
                matchStart, matchEnd, matchCigar, matchSeq, matchChr = matchObject.get()

                for snip in snpData['SNP']:
                    if (snip['chrom'] == matchChr):
                        # print("2. reference_name: " + sequence.reference_name)v
                        # print("2. sequence.query_name: " + sequence.query_name)
                        if( 'rs16947' in snip['name'] and 'NumReads442' in sequence.query_name):
                            print("rs1135822")

                        chromStart = snip['chromStart']
                        chromEnd = snip['chromEnd']

                        if (matchStart < chromStart < matchEnd and matchStart < chromEnd < matchEnd):
                            # print("1. reference_name: " + sequence.reference_name + " | 1. sequence.query_name: " + sequence.query_name)

                            # AmpliconName = sequence.query_name.split('#')[2]

                            #Noise filtering if insertions are more than 10 in the sequence.
                            #Trim that sequence out.

                            insertionCount = callsnip.CigarInsertionCount(matchCigar)
                            if (insertionCount > 10):
                                continue

                            rawReads = sequence.query_name.split('NumReads')[1]
                            targetPositionStart = chromStart - matchStart
                            targetPositionEnd = chromEnd - matchStart
                            print("targetPositionStart : " + str(targetPositionStart) + " | targetPositionEnd: " + str(targetPositionEnd))
                            calStart, calend,baseMark = callsnip.CigarCalculation(targetPositionStart, targetPositionEnd, matchCigar)
                            print("start : " + str(calStart) + " | end: " + str(calend))

                            rawbases = matchSeq[calStart:calend]

                            if(baseMark=='D'):
                                rawbases='-'
                                #for i in range(calend-calStart):
                                #    rawbases=rawbases+'-'

                            if (calStart == calend and rawbases == ''):
                                rawbases = '-'

                            #minBaseOcurrances, minBasePercentages, minBaseQualityPercentage = qualityCalculation.getQuality(qualityData, rawbases, calStart, calend, sequence.query_name)
                            minBaseOcurrances, minBasePercentages, minBaseQualityPercentage = 100,100,100
                            #rawbases = sequence.seq[targetPositionStart:targetPositionEnd]

                            snps = Snp(1, sequence.query_name, rawReads, snip['name'], snip['chrom'].replace("chr", ""),
                                       str(chromStart), str(chromEnd), snip['refNCBI'], snip['refUCSC'],
                                       snip['observed'], rawbases, str(minBaseOcurrances), str(minBasePercentages),
                                       str(minBaseQualityPercentage), sequence)

                            # dt.update(vars(snps))
                            # dt = json.dumps(dt)

                            if (snip['name'] in snip_dic):
                                dicSnipDetail = snip_dic[snip['name']]
                                if (rawbases in dicSnipDetail):
                                    getSnip = dicSnipDetail[rawbases]

                                    getSnip.rawReads = int(getSnip.rawReads) + int(snps.rawReads)
                                    getSnip.minBaseOcurrances = int(getSnip.minBaseOcurrances) + int(snps.minBaseOcurrances)

                                    if (getSnip.minBasePercentages > snps.minBasePercentages):
                                        getSnip.minBasePercentages = snps.minBasePercentages
                                    if (getSnip.minBasePercentages > snps.minBasePercentages):
                                        getSnip.minBasePercentages = snps.minBasePercentages
                                    if (getSnip.minBaseQualityPercentage > snps.minBaseQualityPercentage):
                                        getSnip.minBaseQualityPercentage = snps.minBaseQualityPercentage

                                    # lstSnip.append(snps)
                                    dicSnipDetail[rawbases] = getSnip
                                else:
                                    # lstSnip = []
                                    # lstSnip.append(snps)
                                    dicSnipDetail[rawbases] = snps
                            else:
                                # lstSnip = []
                                dicSnipDetail = {}
                                # lstSnip.append(snps)
                                dicSnipDetail[rawbases] = snps
                                snip_dic[snip['name']] = dicSnipDetail

                            if (snip['name'] in snip_totalReads):
                                snip_totalReads[snip['name']] = int(snip_totalReads[snip['name']])+int(rawReads)
                            else:
                                snip_totalReads[snip['name']] = int(rawReads)


                            #amp.add(snps)
                            #writer.writerow({'experimentName': experimentName, 'sampleId': sampleID, 'seqName': sequence.query_name, 'rawReads': rawReads, 'snpName': snip['name'], 'chrom': snip['chrom'], 'chromStart': str(chromStart), 'chromEnd': str(chromEnd), 'refNCBI': snip['refNCBI'], 'refUCSC': snip['refUCSC'], 'targeted': snip['observed'], 'observed': rawbases, 'minBaseOcurrances': str(minBaseOcurrances), 'minBasePercentages': str(minBasePercentages), 'minBaseQualityPercentage': str(minBaseQualityPercentage)})

                            print(experimentName + ',' + sampleID + ',' + sequence.query_name + ',' + rawReads + ',' + snip[
                                'name'] + ',' + snip['chrom'] + ',' + str(chromStart) + ',' + str(chromEnd) + ',' + snip[
                                      'refNCBI'] + ',' + snip['refUCSC'] + ',' + snip['observed'] + ',' + rawbases  + ',' + str(minBaseOcurrances) + ',' + str(minBasePercentages) + ',' + str(minBaseQualityPercentage))
                            isTargeted = True


                # AmpliconName = sequence.query_name.split('#')[2]
                # amp = Amplicon(AmpliconName)

                # if AmpliconName in AmpliconList:
                #     amp = AmpliconList[AmpliconName] # Get Amplicon object by AmpliconName
                #     amp.add(dt)

                #if isTargeted == True:
                #    smpl.add(amp)
                #else:
                #    print("None Target List: " + sequence.query_name)

final_snp_list = callsnip.filterSnipObject(snip_dic, snip_totalReads, trimPercent)

############### CNV Logic

isPassed = 0

for cnv in cnvData:
    if(cnv['ExperimentName'] == experimentName and cnv['ClientSampleID'] == sampleID):
        for snip in snpData['SNP']:
            if( cnv['SNPName'] == snip['name'] ):
                snps = Snp(1, '', cnv['CNV'], cnv['SNPName'], snip['chrom'].replace("chr", ""), snip['chromStart'], snip['chromEnd'],
                           snip['refNCBI'], snip['refUCSC'], snip['observed'], '<CNV>',
                           str(0), str(100), str(100), '')

                dicSnipDetail = {}
                # lstSnip.append(snps)
                dicSnipDetail['CNV'] = snps
                final_snp_list[cnv['SNPName']] = dicSnipDetail
                isPassed = 1
                break

    if(isPassed == 1):
        break


#########################


write.csvWriter(outputpath, final_snp_list, experimentName,sampleID)

# print("sampleID", sampleID)
# for name in sorted(final_snp_list):
#     print("name", name)
#     for base, snp in final_snp_list[name].items():
#         print(base, snp.rawReads, snp.snpName, snp.chrom, snp.chromStart, snp.chromEnd, snp.refNCBI, snp.refUCSC, snp.observed, snp.rawBases)


converter = runConvert.Converter()
vcfs = converter.convert(final_snp_list, sampleID)
header = reader.VCFReader(args.vcfpath, sampleID)

write.TextWriter(outputpath_vcf, header)
write.VCFWriter(outputpath_vcf, vcfs)



#autoResult.add(smpl)

#autoResult.autoTyping_dic = {}
#autoResult.autoTyping_dic["snpVersion"] = autoResult.snpVersion
#autoResult.autoTyping_dic["targetSnpVersion"] = autoResult.targetSnpVersion
#autoResult.autoTyping_dic["sample"] = autoResult.sample_list

#Convert_Json = {
#    "autoTypingResult": autoResult.autoTyping_dic
#}

#jsonFormat = json.dumps(Convert_Json, indent=4)
#print(jsonFormat)




import copy

class PacBioObject:

    def __init__(self, jobName, experimentName, sampleID, barcodeID, experimentID):
        self.jobName = jobName
        self.experimentName = experimentName
        self.sampleID = sampleID
        self.barcodeID = barcodeID
        self.experimentID = experimentID


    def reprJSON(self):
        return dict(jobName=self.jobName, experimentName=self.experimentName, sampleID=self.sampleID, barcodeID=self.barcodeID, experimentID=self.experimentID)

def make_PacBioObject(jobName, experimentName, sampleID, barcodeID,experimentID):
    pacBioObject = PacBioObject(jobName, experimentName, sampleID, barcodeID,experimentID)
    return pacBioObject



import argparse
import os.path
import json
import csv
from pathlib import Path
from WholeGenomeAnalysis.SNP.model.PacBioObject import PacBioObject

def remove_non_ascii(text):
    return ''.join(i for i in text if ord(i)<128)

parser = argparse.ArgumentParser(description='Input for Snip program')

parser.add_argument("-i", "--inputcsv", help='input csv file with tab delimited', required=True, dest="inputcsv")
parser.add_argument("-o", "--output", help='output location', required=True, dest="output")

args = parser.parse_args()

with open(args.inputcsv, newline='') as csv_file:
    sequence_reader = csv.reader(csv_file, delimiter='\t')
    for line in sequence_reader:
        ExperimentName = remove_non_ascii(line[0])
        SampleName = line[1]
        BarcodeID = line[2]
        SequenceName = line[3]
        Sequence = line[4]
        ExperimentID = line[6]
        jName = line[7]
        experiment_foder = Path(args.output+'/'+ExperimentName+'/')
        data_folder = Path(args.output+'/'+ExperimentName+'/'+SampleName+'/')
        File_Path = Path(args.output + '/'+ExperimentName+'/' + SampleName + '/' + SampleName + '.fasta')
        jSon_Path = Path(args.output + '/'+ExperimentName+'/' + SampleName + '/' + SampleName + '.json')
        #file_to_open = data_folder / SampleName+'.fastq'

        #if "PGx" not in SampleName:
        #    continue

        if not os.path.exists(experiment_foder):
            os.mkdir(experiment_foder)

        if not os.path.exists(data_folder):
            os.mkdir(data_folder)

        #if not os.path.exists(jSon_Path):
        #    os.mkdir(data_folder)
        pbo = PacBioObject(jName, ExperimentName, SampleName, BarcodeID, ExperimentID)

        outF = open(File_Path, "a")
        outF.write('>'+SequenceName+"\n")
        outF.write(Sequence+"\n")
        outF.close()
        import json

        #with open(jSon_Path, 'w', encoding='utf-8-sig') as f:
        #    json.dump(pbo.__dict__, f, ensure_ascii=False)


        with open(jSon_Path, 'w', encoding='utf-8') as f:
            json.dump(pbo.__dict__, f, ensure_ascii=False, indent=4)

        #outF = open("myOutFile.txt", "w")




import json
import datetime

def JsonReader(filePath):
    with open(filePath) as json_file:
        return json.load(json_file)

def VCFReader(filePath, sampleid):
    header = []

    with open(filePath, 'r') as v:
        for line in v:

            if "fileDate" in line:
                date = datetime.datetime.today()
                line = line.replace('\n', '') + date.strftime('%Y%m%d') + '\n'

            elif "CHROM" in line:
                line = line + '\t' + sampleid + '\n'

            header.append(line)

    return header


from setuptools import setup, find_packages

setup(
    name                = 'snpcaller',
    version             = '0.2.6',
    description         = 'Read bam file and find the snp positions',
    author              = 'seho, sungkook',
    author_email        = 'seho@histogenetics.com',
    url                 = 'https://github.com/Histogenetics/GenomeAnalysis',
    download_url        = 'https://github.com/Histogenetics/GenomeAnalysis/archive/master.zip',
    install_requires    =  [],
    packages            = find_packages(exclude = []),
    #packages=['WholeGenomeAnalysis','WholeGenomeAnalysis.SNP','WholeGenomeAnalysis.SNP.process'],
    keywords            = ['snp'],
    python_requires     = '>=3',
    package_data        = {},
    zip_safe            = False,
    classifiers         = [
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
)


def getSnipPosition(bam,jsondata):
    for sequence in bam:
        if (hasattr(sequence, 'reference_name')):
            print(sequence.reference_name)
            for snp in jsondata['SNP']:
                if(snp['chrom'] == sequence.reference_name):
                    print


    for sequence in bam:
        if (hasattr(sequence, 'reference_name')):
            start=sequence.pos
            end=sequence.pos+len(sequence.seq)
            for snp in data['SNP']:
                if(snp['chrom'] == sequence.reference_name):
                    chromStart=snp['chromStart']
                    chromEnd=snp['chromEnd']
                    if(start < chromStart <  end and start < chromEnd < end):
                        AmpliconName=sequence.query_name.split('#')[2]
                        rawReads = sequence.query_name.split('#')[4]
                        targetPositionStart=chromStart-start
                        targetPositionEnd=chromEnd-start

                        rawbases=sequence.seq[targetPositionStart:targetPositionEnd]
                        print(BlotName+','+SampleName+','+AmpliconName+','+rawReads+','+snp['name']+','+snp['chrom']+','+str(chromStart)+','+str(chromEnd)+','+snp['refNCBI']+','+snp['refUCSC']+','+snp['observed']+','+rawbases)

def CigarCalculation(targetPositionStart, targetPositionEnd, cigar):
    calPositionStart = 1
    returnStart = targetPositionStart
    returnEnd = targetPositionEnd

    targetBaseCount = targetPositionEnd - targetPositionStart

    baseMark = 'M'

    #if(len(cigar) > 1):
    #    print(cigar)
    # Cigar value 0 is mutation, 1 is Insertion , 2 is Deletion.
    for arrCigar in cigar:
        #Check is Instance because sometimes bamnostic return (('BAM_CMATCH',0),1) like this.
        if(isinstance(arrCigar[0],list)):
            mutationType=arrCigar[0][1]
        else:
            mutationType=arrCigar[0]

        value=arrCigar[1]

        if(returnStart == calPositionStart and targetBaseCount == 0):
            if (mutationType == 1):
                # returnStart = targetPositionStart + value
                returnEnd = returnEnd + value
                calPositionStart = calPositionStart + value
                baseMark='I'

            return returnStart, returnEnd, baseMark
        elif (returnStart+1 == calPositionStart and targetBaseCount == value and mutationType == 2):
            if (mutationType == 2):
                # returnStart = targetPositionStart + value
                baseMark = 'D'

            return returnStart, returnEnd, baseMark
        elif(returnStart > calPositionStart):
            if(mutationType==0):
                calPositionStart=calPositionStart+value
            elif(mutationType==1):
                returnStart = returnStart + value
                returnEnd = returnEnd + value
                calPositionStart = calPositionStart+value
            elif(mutationType==2):
                returnStart=returnStart-value
                returnEnd=returnEnd-value
                #calPositionStart=calPositionStart-value
        else:
            return returnStart,returnEnd,baseMark

    return returnStart, returnEnd, baseMark

def CigarInsertionCount(cigar):
    # Cigar value 0 is mutation, 1 is Insertion , 2 is Deletion.

    insertionCount = 0
    for arrCigar in cigar:
        if(isinstance(arrCigar[0],list)):
            mutationType=arrCigar[0][1]
        else:
            mutationType=arrCigar[0]

        if (mutationType == 1):
            insertionCount = insertionCount + 1

    return insertionCount

def filterSnipObject(snip_dic,snip_totalReads, trimpercentage):
    final_snp_list = {}
    snip_result = {}
    # Filter Rawreads by Cut Off
    for snip, snip_observe  in snip_dic.items():
        cutoff = snip_totalReads[snip] * trimpercentage
        for observe, snp in snip_observe.items():
            if(float(snp.rawReads) < cutoff):
                snp.isValid = 0

    for snip, snip_observe  in snip_dic.items():
        # If snip position is more than two, we will not allow to have
        #if (len(snip_observe.keys()) < 3):
        for observe, snp in snip_observe.items():
            validCount=0
            for tempobserve, tempsnp in snip_observe.items():
                if(tempsnp.isValid==1):
                    validCount = validCount+1
                else:
                    print("isValid == 0 : " + str(tempsnp.seqName))

            if(validCount > 2):
                continue

            #If It is not valid sequence, It is skipped.
            if(snp.isValid == 0):
                continue

            if (snip in final_snp_list):
                dicSnipDetail = final_snp_list[snip]
                if (observe in dicSnipDetail):
                    getSnip = dicSnipDetail[observe]
                    dicSnipDetail[observe] = getSnip

                else:
                    dicSnipDetail[observe] = snp
            else:
                dicSnipDetail = {}
                dicSnipDetail[observe] = snp
                final_snp_list[snip] = dicSnipDetail

    return final_snp_list



